#!/usr/bin/perl -w

#
# Delete all entries which fall inside the restriction.
# These happen when you add new restrictions.
#
# Keep running this script until there are no more entries reported
# to be deleted.
#

use strict;
use DBI;
use FindBin qw($Bin);

my $DBDIR="";
{
    my @a=split("/",$Bin);
    pop(@a);
    $DBDIR=join("/",@a)."/db";
}

my $dbh=DBI->connect("dbi:SQLite:dbname=$DBDIR/newports.db","","");

my %restrictions=();

{
    my $sth=$dbh->prepare("
	SELECT	directory,restriction
	FROM	restrictports
    ");
    $sth->execute();
    while (my @a=$sth->fetchrow_array()) {
	$restrictions{$a[0]}=$a[1];
    }
}

my %tobedeleted=();
{
    my $sth_select=$dbh->prepare("
	SELECT  newversion
	FROM	newports
	WHERE	directory=?
    ");
    foreach my $dir (sort(keys(%restrictions))) {
	$sth_select->execute($dir);
	while (my @a=$sth_select->fetchrow_array()) {
	    next if (substr($a[0],0,length($restrictions{$dir})) eq $restrictions{$dir});
	    $tobedeleted{$dir}{$a[0]}=1;
	}
    }
}

{
    my $sth_delete=$dbh->prepare("
	DELETE FROM newports
	WHERE	directory=?
	AND	newversion=?
    ");
    foreach my $dir (sort(keys(%tobedeleted))) {
	foreach my $v (sort(keys(%{$tobedeleted{$dir}}))) {
	    my $c=$sth_delete->execute($dir,$v);
	    print "Removing $dir - version $v - $c entries\n";
	}
    }
}

$dbh->disconnect;
