#!/usr/bin/perl -w

use strict;
use LWP;
use LWP::UserAgent;
use Data::Dumper;
use DBI;
use POSIX;
use FindBin qw($Bin);
use Getopt::Std;

my %opts;
getopts("f",\%opts);

my $FORCE=0;
$FORCE=1 if (defined $opts{f});

my $MAXTRIES=4;

my $DBDIR="";
my $directory="";
{
    my @a=split("/",getcwd());
    $directory="$a[-2]/$a[-1]";

    @a=split("/",$Bin);
    pop(@a);
    $DBDIR=join("/",@a)."/db";
}
print "directory: $directory\n";

# stop buffering of stdout
$|=1;

my $dbh=DBI->connect("dbi:SQLite:dbname=$DBDIR/newports.db","","");

#
# Find reasons why we shouldn't run.
#
{
    # check if directory already done
    my $sth=$dbh->prepare("
	SELECT	time
	FROM	checkeddirectories
	WHERE	directory=?
    ");
    $sth->execute($directory);
    my @a=$sth->fetchrow_array();
    if ($sth->rows!=0) {
	if (!$FORCE) {
	    print "Directory $directory already checked on $a[0], use -f to force\n";
	    exit;
	} else {
	    print "Directory $directory already checked on $a[0], checking anyway\n";
	    my $sth=$dbh->prepare("
		DELETE FROM	checkeddirectories
		WHERE	directory=?
	    ");
	    $sth->execute($directory);
	}
    }
}

# initialize the database
#
# create table failedhosts ( hostname text, protocol text, times int );
# create table restrictports ( directory text, restriction text );
# create table ignoreports ( category text, port text );
# create table ignorehosts ( hostname text, protocol text, reason text );
# create table ignoremaintainers ( maintainer text );
# create table newports ( directory text, category text, port text,
#			currentversion text, newversion text,
#			url text, maintainer text, reported text );
# create table checkeddirectories ( directory text, time text );
#
my $sth_failedhosts_insert=$dbh->prepare("
	INSERT INTO failedhosts(hostname,protocol,times)
	values(?,?,0)
");
my $sth_failedhosts_update=$dbh->prepare("
	UPDATE	failedhosts
	SET	times=times+1
	WHERE	hostname=?
	AND	protocol=?
");
my $sth_failedhosts_delete=$dbh->prepare("
	DELETE FROM	failedhosts
	WHERE	hostname=?
	AND	protocol=?
");
my $sth_newports_delete=$dbh->prepare("
	DELETE FROM newports
	WHERE	directory=?
	AND	currentversion!=?
");
my $sth_newports_insert=$dbh->prepare("
	INSERT INTO newports(directory,category,port,
			currentversion,newversion,
			url,maintainer,reported)
	values(?,?,?,?,?,?,?,?)
");
my $sth_newports_find=$dbh->prepare("
	SELECT	count(*)
	FROM	newports
	WHERE	directory=?
	AND	currentversion=?
	AND	newversion=?
	AND	url=?
	AND	maintainer like ?
");


my %alreadydone=();
my %restrictports=();
my %ignorehosts=();
my %ignoreports=();
my %ignoremaintainers=();
my %failedhosts=();
{
    my $sth;

    $sth=$dbh->prepare("
	SELECT	url
	FROM	newports
	WHERE	directory=?
    ");
    $sth->execute($directory);
    while (my @a=$sth->fetchrow_array()) {
	$alreadydone{$a[0]}=1;
    }

    $sth=$dbh->prepare("
	SELECT	hostname,protocol,times
	FROM	failedhosts
    ");
    $sth->execute();
    while (my @a=$sth->fetchrow_array()) {
	$failedhosts{$a[0]}{$a[1]}=1*$a[2];
    }

    $sth=$dbh->prepare("
	SELECT	hostname,protocol,times
	FROM	failedhosts
    ");
    $sth->execute();
    while (my @a=$sth->fetchrow_array()) {
	$failedhosts{$a[0]}{$a[1]}=1*$a[2];
    }

    $sth=$dbh->prepare("
	SELECT	directory,restriction
	FROM	restrictports
    ");
    $sth->execute();
    while (my @a=$sth->fetchrow_array()) {
	$restrictports{$a[0]}=$a[1];
    }

    $sth=$dbh->prepare("
	SELECT	maintainer
	FROM	ignoremaintainers
    ");
    $sth->execute();
    while (my @a=$sth->fetchrow_array()) {
	$ignoremaintainers{lc($a[0])}=1;
    }

    $sth=$dbh->prepare("
	SELECT	hostname,protocol
	FROM	ignorehosts
    ");
    $sth->execute();
    while (my @a=$sth->fetchrow_array()) {
	$ignorehosts{$a[0]}{$a[1]}=1;
    }

    $sth=$dbh->prepare("
	SELECT	category,port
	FROM	ignoreports
    ");
    $sth->execute();
    while (my @a=$sth->fetchrow_array()) {
	$ignoreports{$a[0]}{$a[1]}=1;
    }
}

# initiatilize LWP
my $ua=LWP::UserAgent->new;
$ua->agent("New Ports File Checker - http://www.mavetju.org/newportsfilechecker.php");
$ua->{requests_redirectable}=0;
$ua->{timeout}=15;

# get some random information from the port
my $PORTNAME=`make -V PORTNAME`;
chomp($PORTNAME);
print "PORTNAME: $PORTNAME\n";

my $PORTVERSION=`make -V PORTVERSION`;
chomp($PORTVERSION);
print "PORTVERSION: $PORTVERSION\n";
my @PORTVERSIONS=();
{
    # make list of all newer versions:
    # 1.2.3 -> 1.2.4, 1.3.0, 2.0.0
    my @a=split(/\./,$PORTVERSION);
    for (my $i=$#a;$i>=0;$i--) {
	my @b=@a;
	$b[$i]++;
	for (my $j=$i+1;$j<=$#a;$j++) {
	    $b[$j]=0;
	}
	my $p=join(".",@b);
	next
	    if (defined $restrictports{$directory}
	    && substr($p,0,length($restrictports{$directory})) ne $restrictports{$directory});
	push(@PORTVERSIONS,$p);
    }
}
print "PORTVERSIONS: @PORTVERSIONS\n";

{
    # remove entries which don't have the current version number in it.
    my $rows=$sth_newports_delete->execute($directory,$PORTVERSION);
    print "Removed previous version ($rows entries)\n"
	if (defined $rows && $rows ne "0E0");
    $sth_newports_delete->finish();
}

my $MAINTAINER=`make -V MAINTAINER`;
chomp($MAINTAINER);
print "MAINTAINER: $MAINTAINER\n";
if (defined $ignoremaintainers{lc($MAINTAINER)}) {
    print "Maintainer $MAINTAINER is ignored\n";
    exit;
}

my $CATEGORIES=`make -V CATEGORIES`;
chomp($CATEGORIES);
my @CATEGORIES=split(" ",$CATEGORIES);
print "CATEGORIES: @CATEGORIES\n";
my $CATEGORY=$CATEGORIES[0];

my $DISTFILES=`make -V DISTFILES`;
chomp($DISTFILES);
my @DISTFILES=split("\n",$DISTFILES);
print "DISTFILES: @DISTFILES\n";

my $FETCHFILES=`MASTER_SITE_OVERRIDE= MASTER_SITE_BACKUP= make fetch-all-list`;
chomp($FETCHFILES);
my @FETCHFILES=split("\n",$FETCHFILES);
print "FETCHFILES: @FETCHFILES\n";

my %TRYFILES=();
foreach my $ff (@FETCHFILES) {
    next if (index($ff,$PORTVERSION)==-1);
    # make the new files to fetch:
    # foo-1.2.3.tar.gz -> foo-1.2.4.tar.gz, foo-1.3.0.tar.gz, foo-2.0.0.tar.gz
    foreach my $pv (@PORTVERSIONS) {
	my $tf=$ff;
	$tf=~s/$PORTVERSION/$pv/g;
	$TRYFILES{$tf}=$pv;

	my @tf=split(/\./,$pv);
	my @pv=split(/\./,$PORTVERSION);
	while ($#tf>1) {
	    pop(@tf);
	    pop(@pv);
	    my $_tf=join(".",@tf);
	    my $_pv=join(".",@pv);

	    $tf=$ff;
	    $tf=~s/$PORTVERSION/$pv/g;
	    $tf=~s/$_pv/$_tf/g;
	    $TRYFILES{$tf}=$pv;
	}
    }
}
print "TRYFILES:\n",join("\n",keys(%TRYFILES)),"\n";

# See if the port has to be ignored
foreach my $cat (@CATEGORIES) {
    if (defined $ignoreports{$cat}{$PORTNAME}) {
	print "Port $cat/$PORTNAME is ignored\n";
	exit;
    }
}

my @FOUNDFILES=();
foreach my $tf (keys(%TRYFILES)) {

    if ($tf!~/([a-z]+):\/\/(.*?)\//) {
	print "Not an URL: $tf\n";
	next;
    }
    my $hostname=$2;
    my $protocol=$1;
    if (defined $ignorehosts{$hostname}{$protocol}) {
	print "Site $protocol://$hostname/ is ignored\n";
	next;
    }
    if (defined $failedhosts{$hostname}{$protocol} &&
	$failedhosts{$hostname}{$protocol}>$MAXTRIES) {
	print "Site $protocol://$hostname/ is marked as failed\n";
	next;
    }
    if (defined $alreadydone{$tf}) {
	print "URL $tf is already found\n";
	next;
    }

    my $req=HTTP::Request->new(HEAD => $tf);
    print "Trying: $tf ==> ";
    my $res=$ua->request($req);
    if ($res->is_success) {
	my $rc=$res->{_rc};
	if ($rc eq "200") {
	    print "200\n";
	    if ($res->{"_headers"}{"content-type"}=~/^text\//) {
		print "Content-type ",$res->{"_headers"}{"content-type"}," ignored\n";
		next;
	    }
	    push(@FOUNDFILES,$tf);
	    $sth_newports_find->execute($directory,"$PORTVERSION","$TRYFILES{$tf}",$tf,$MAINTAINER);
	    my @a=$sth_newports_find->fetchrow_array();
	    if ($a[0] ne "0") {
		print "$tf already in the list\n";
		next;
	    }
	    $sth_newports_find->finish();
	    $sth_newports_insert->execute($directory,$CATEGORY,$PORTNAME,"$PORTVERSION","$TRYFILES{$tf}",$tf,lc($MAINTAINER),0);
	    print "Added $tf to the list\n";

	    # it was successful this time
	    if (defined $failedhosts{$hostname}{$protocol}) {
		$sth_failedhosts_delete->execute($hostname,$protocol);
		print "Removing $protocol://$hostname/ from failed host list\n";
	    }
	}
    } else {
	print $res->status_line,"\n";
	my $rc=$res->{_rc};
	if ($rc eq "500") {
	    if (!defined $failedhosts{$hostname}{$protocol}) {
		$sth_failedhosts_insert->execute($hostname,$protocol);
		print "Adding $protocol://$hostname/ to failed host list\n";
	    } else {
		$sth_failedhosts_update->execute($hostname,$protocol);
		print "Updating $protocol://$hostname/ in failed host list\n";
	    }
	    $failedhosts{$hostname}{$protocol}=1;
	} else {
	    # it was successful this time
	    if (defined $failedhosts{$hostname}{$protocol}) {
		$sth_failedhosts_delete->execute($hostname,$protocol);
		print "Removing $protocol://$hostname/ from failed host list\n";
	    }
	}
    }
}

{
    my $sth=$dbh->prepare("
	INSERT INTO checkeddirectories(directory,time)
	values(?,?)
    ");
    $sth->execute($directory,POSIX::strftime("%Y-%m-%d %H:%M:%S",localtime));
    $sth->finish();
}

print "FOUNDFILES: @FOUNDFILES\n\n";

$sth_failedhosts_insert->finish();
$sth_failedhosts_delete->finish();
$sth_failedhosts_update->finish();
$sth_newports_insert->finish();
$sth_newports_find->finish();
$dbh->disconnect();
